@@ -245,11 +245,13 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		xc->pan.val = fxp;
 		break;
 	case FX_OFFSET:		/* Set sample offset */
+		EFFECT_MEMORY(fxp, xc->offset.memory);
 		SET(OFFSET);
-		if (fxp) {
-			xc->offset_val = xc->offset = fxp << 8;
-		} else {
-			xc->offset_val = xc->offset;
+		if (note) {
+			xc->offset.val = xc->offset.val2 = fxp << 8;
+		}
+		if (e->ins) {
+			xc->offset.val2 = fxp << 8;
 		}
 		break;
 	case FX_VOLSLIDE:	/* Volume slide */
@@ -292,16 +294,12 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			xc->vol.memory = fxp;
 			h = MSN(fxp);
 			l = LSN(fxp);
-			if (HAS_QUIRK(QUIRK_VOLPDN)) {
-				if (fxp)
+			if (fxp) {
+				if (HAS_QUIRK(QUIRK_VOLPDN)) {
 					xc->vol.slide = l ? -l : h;
-			} else {
-				if (l == 0)
-					xc->vol.slide = h;
-				else if (h == 0)
-					xc->vol.slide = -l;
-				else
-					RESET(VOL_SLIDE);
+				} else {
+					xc->vol.slide = h ? h : -l;
+				}
 			}
 		}
 
@@ -322,12 +320,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		if (fxp) {
 			h = MSN(fxp);
 			l = LSN(fxp);
-			if (l == 0)
-				xc->vol.slide2 = h;
-			else if (h == 0)
-				xc->vol.slide2 = -l;
-			else
-				RESET(VOL_SLIDE_2);
+			xc->vol.slide2 = h ? h : -l;
 		}		
 		break;
 	case FX_JUMP:		/* Order jump */
@@ -373,6 +366,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			if (fxp == 0) {
 				/* mark start of loop */
 				f->loop[chn].start = p->row;
+				if (HAS_QUIRK(QUIRK_FT2LOOP))
+				  p->flow.jumpline = p->row;
 			} else {
 				/* end of loop */
 				if (f->loop[chn].count) {
@@ -535,11 +530,11 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 					xc->gvol.slide = 0;
 					xc->gvol.fslide = -l;
 				} else {
-					xc->gvol.slide = h - l;
+					xc->gvol.slide = h ? h : -l;
 					xc->gvol.fslide = 0;
 				}
 			} else {
-				xc->gvol.slide = h - l;
+				xc->gvol.slide = h ? h : -l;
 				xc->gvol.fslide = 0;
 			}
 		} else {
@@ -558,11 +553,13 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		xc->p_idx = fxp;
 		break;
 	case FX_PANSLIDE:	/* Pan slide (XM) */
-		/* TODO: add memory */
+		EFFECT_MEMORY(fxp, xc->pan.memory);
 		SET(PAN_SLIDE);
-		if (fxp) {
-			xc->pan.slide = LSN(fxp) - MSN(fxp);
-		}
+		xc->pan.slide = LSN(fxp) - MSN(fxp);
+		break;
+	case FX_PANSL_NOMEM:	/* Pan slide (XM volume column) */
+		SET(PAN_SLIDE);
+		xc->pan.slide = LSN(fxp) - MSN(fxp);
 		break;
 
 #ifndef LIBXMP_CORE_DISABLE_IT
@@ -641,19 +638,18 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 				goto fx_trk_fvslide;
 			}
 		}
-		SET(TRK_VSLIDE);
 
+		SET(TRK_VSLIDE);
 		if (fxp) {
 			h = MSN(fxp);
 			l = LSN(fxp);
 
 			xc->trackvol.memory = fxp;
-			if (l == 0)
-				xc->trackvol.slide = h;
-			else if (h == 0)
-				xc->trackvol.slide = -l;
-			else
-				RESET(TRK_VSLIDE);
+			if (HAS_QUIRK(QUIRK_VOLPDN)) {
+				xc->trackvol.slide = l ? -l : h;
+			} else {
+				xc->trackvol.slide = h ? h : -l;
+			}
 		}
 
 		break;
@@ -755,12 +751,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		if (fxp) {
 			h = MSN(fxp);
 			l = LSN(fxp);
-			if (l == 0)
-				xc->vol.fslide = h;
-			else if (h == 0)
-				xc->vol.fslide = -l;
-			else
-				RESET(FINE_VOLS);
+			xc->vol.fslide = h ? h : -l;
 		}		
 		break;
 	case FX_NSLIDE_DN:
