@@ -18,7 +18,7 @@ char **format_list(void);
 #define NUM_FORMATS 52
 #define NUM_PW_FORMATS 39
 
-int pw_test_format(FILE *, char *, const int, struct xmp_test_info *);
+int pw_test_format(HIO_HANDLE *, char *, const int, struct xmp_test_info *);
 #endif
 
 #endif
