@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,17 +22,26 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef LIBXMP_CORE_PLAYER
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#endif
 #include <limits.h>
 #include <errno.h>
 #include "common.h"
 #include "hio.h"
 #include "mdataio.h"
 
+static long get_size(FILE *f)
+{
+	long size, pos;
+
+	pos = ftell(f);
+	if (pos >= 0) {
+		fseek(f, 0, SEEK_END);
+		size = ftell(f);
+		fseek(f, pos, SEEK_SET);
+		return size;
+	} else {
+		return pos;
+	}
+}
 
 int8 hio_read8s(HIO_HANDLE *h)
 {
@@ -191,6 +200,8 @@ HIO_HANDLE *hio_open(void *path, char *mode)
 	if (h->handle.file == NULL)
 		goto err2;
 
+	h->size = get_size(h->handle.file);
+
 	return h;
 
     err2:
@@ -209,6 +220,7 @@ HIO_HANDLE *hio_open_mem(void *ptr, long size)
 	
 	h->type = HIO_HANDLE_TYPE_MEMORY;
 	h->handle.mem = mopen(ptr, size);
+	h->size = size;
 
 	return h;
 }
@@ -222,7 +234,8 @@ HIO_HANDLE *hio_open_file(FILE *f)
 		return NULL;
 	
 	h->type = HIO_HANDLE_TYPE_FILE;
-	h->handle.file = fdopen(fileno(f), "rb");
+	h->handle.file = f /*fdopen(fileno(f), "rb")*/;
+	h->size = get_size(f);
 
 	return h;
 }
@@ -246,37 +259,7 @@ int hio_close(HIO_HANDLE *h)
 	return ret;
 }
 
-#ifndef LIBXMP_CORE_PLAYER
-
-HIO_HANDLE *hio_open_fd(int fd, char *mode)
+long hio_size(HIO_HANDLE *h)
 {
-	HIO_HANDLE *h;
-	
-	h = (HIO_HANDLE *)malloc(sizeof (HIO_HANDLE));
-	if (h == NULL)
-		return NULL;
-	
-	h->type = HIO_HANDLE_TYPE_FILE;
-	h->handle.file = fdopen(fd, mode);
-	if (h->handle.file == NULL) {
-		free(h);
-		return NULL;
-	}
-
-	return h;
+	return h->size;
 }
-
-int hio_stat(HIO_HANDLE *h, struct stat *st)
-{
-	switch (HIO_HANDLE_TYPE(h)) {
-	case HIO_HANDLE_TYPE_FILE:
-		return fstat(fileno(h->handle.file), st);
-	case HIO_HANDLE_TYPE_MEMORY:
-		return mstat(h->handle.mem, st);
-	default:
-		return -1;
-	}
-}
-
-#endif
-
