@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -78,6 +78,13 @@ const struct module_quirk mq[] = {
 		XMP_FLAGS_VBLANK
 	},
 
+	/* Another version of Klisje paa klisje sent by Steve Fernandez */
+	{
+		{ 0x12, 0x19, 0x1c, 0x90, 0x41, 0xe3, 0xfd, 0x70,
+		  0xb7, 0xe6, 0xb3, 0x94, 0x8b, 0x21, 0x07, 0x63 },
+		XMP_FLAGS_VBLANK
+	},
+
 	{
 		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
@@ -128,39 +135,6 @@ int exclude_match(char *name)
 	return 0;
 }
 
-int get_temp_dir(char *buf, int size)
-{
-#if defined WIN32
-	const char def[] = "C:\\WINDOWS\\TEMP";
-	char *tmp = getenv("TEMP");
-
-	strncpy(buf, tmp ? tmp : def, size);
-	strncat(buf, "\\", size);
-#elif defined __AMIGA__
-	strncpy(buf, "T:", size);
-#elif defined ANDROID
-#define APPDIR "/sdcard/Xmp for Android"
-	struct stat st;
-	if (stat(APPDIR, &st) < 0) {
-		if (mkdir(APPDIR, 0777) < 0)
-			return -1;
-	}
-	if (stat(APPDIR "/tmp", &st) < 0) {
-		if (mkdir(APPDIR "/tmp", 0777) < 0)
-			return -1;
-	}
-	strncpy(buf, APPDIR "/tmp/", size);
-#else
-	const char def[] = "/tmp";
-	char *tmp = getenv("TMPDIR");
-
-	strncpy(buf, tmp ? tmp : def, size);
-	strncat(buf, "/", size);
-#endif
-
-	return 0;
-}
-
 #endif /* LIBXMP_CORE_PLAYER */
 
 char *adjust_string(char *s)
