@@ -58,6 +58,7 @@ struct mod_magic {
 #define TRACKER_FLEXTRAX	8
 #define TRACKER_MODSGRAVE	9
 #define TRACKER_SCREAMTRACKER3	10
+#define TRACKER_OPENMPT		11
 #define TRACKER_UNKNOWN_CONV	95
 #define TRACKER_CONVERTEDST	96
 #define TRACKER_CONVERTED	97
@@ -148,6 +149,17 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
     hio_seek(f, start + 20, SEEK_SET);
     for (i = 0; i < 31; i++) {
 	hio_seek(f, 22, SEEK_CUR);		/* Instrument name */
+
+	/* OpenMPT can create mods with large samples */
+	hio_read16b(f);				/* sample size */
+	if (hio_read8(f) & 0xf0)		/* test finetune */
+		return -1;
+	if (hio_read8(f) > 0x40)		/* test volume */
+		return -1;
+	hio_read16b(f);				/* loop start */
+	hio_read16b(f);				/* loop size */
+
+#if 0
 	if (hio_read16b(f) & 0x8000)		/* test length */
 		return -1;
 	if (hio_read8(f) & 0xf0)		/* test finetune */
@@ -158,6 +170,7 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
 		return -1;
 	if (hio_read16b(f) & 0x8000)		/* test loop size */
 		return -1;
+#endif
     }
 
     /* Test for UNIC tracker modules
@@ -242,6 +255,7 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     int ptkloop = 0;			/* Protracker loop */
     int tracker_id = TRACKER_PROTRACKER;
     int has_loop_0 = 0;
+    int has_vol_in_empty_ins = 0;
 
     LOAD_INIT();
 
@@ -319,6 +333,11 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	if (subinstrument_alloc(mod, i, 1) < 0)
 	    return -1;
 
+	if (mh.ins[i].size >= 0x8000) {
+	    tracker_id = TRACKER_OPENMPT;
+	    detected = 1;
+	}
+
 	mod->xxs[i].len = 2 * mh.ins[i].size;
 	mod->xxs[i].lps = 2 * mh.ins[i].loop_start;
 	mod->xxs[i].lpe = mod->xxs[i].lps + 2 * mh.ins[i].loop_size;
@@ -391,11 +410,18 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     /* Check if has instruments with loop size 0 */
     for (i = 0; i < 31; i++) {
-	if (mh.ins[i].loop_size == 0)
+	if (mh.ins[i].loop_size == 0) {
+            has_loop_0 = 1;
 	    break;
+	}
     }
-    if (i < 31) {
-        has_loop_0 = 1;
+
+    /* Check if has instruments with size 0 and volume > 0 */
+    for (i = 0; i < 31; i++) {
+	if (mh.ins[i].size == 0 && mh.ins[i].volume > 0) {
+	    has_vol_in_empty_ins = 1;
+	    break;
+	}
     }
 
     /* Test Protracker-like files
@@ -407,7 +433,7 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	 * Protracker effects and Noisetracker restart byte */
         tracker_id = TRACKER_PROBABLY_NOISETRACKER;
     } else if (mh.restart < 0x7f) {
-	if (mod->chn == 4) {
+	if (mod->chn == 4 && !has_vol_in_empty_ins) {
 	    tracker_id = TRACKER_NOISETRACKER;
 	} else {
 	    tracker_id = TRACKER_UNKNOWN;
@@ -444,7 +470,9 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		    if (mh.ins[i].size == 0 && mh.ins[i].loop_size == 1) {
 			switch (mod->chn) {
 			case 4:
-			    tracker_id = TRACKER_NOISETRACKER;	/* or Octalyser */
+			    tracker_id = has_vol_in_empty_ins ?
+				TRACKER_OPENMPT :
+				TRACKER_NOISETRACKER;	/* or Octalyser */
 			    break;
 			case 6:
 			case 8:
@@ -604,6 +632,10 @@ skip_test:
 	tracker = "Protracker clone";
 	m->quirk &= ~QUIRK_MODRNG;
 	break;
+    case TRACKER_OPENMPT:
+	tracker = "OpenMPT";
+	ptkloop = 1;
+	break;
     default:
     case TRACKER_UNKNOWN_CONV:
     case TRACKER_UNKNOWN:
@@ -664,12 +696,15 @@ skip_test:
 	}
     }
 
-    if (mod->chn > 4) {
+    if (tracker_id == TRACKER_PROTRACKER || tracker_id == TRACKER_OPENMPT) {
+	m->quirk |= QUIRK_PROTRACK;
+    } else if (tracker_id == TRACKER_SCREAMTRACKER3) {
+	m->quirk |= QUIRKS_ST3;
+	m->read_event_type = READ_EVENT_ST3;
+    } else if (mod->chn > 4) {
 	m->quirk &= ~QUIRK_MODRNG;
 	m->quirk |= QUIRKS_FT2;
 	m->read_event_type = READ_EVENT_FT2;
-    } else if (strcmp(tracker, "Protracker") == 0) {
-	m->quirk |= QUIRK_INVLOOP;
     }
 
     return 0;
