@@ -9,7 +9,7 @@
 #include "prowiz.h"
 
 
-static int depack_skyt(FILE *in, FILE *out)
+static int depack_skyt(HIO_HANDLE *in, FILE *out)
 {
 	uint8 c1, c2, c3, c4;
 	uint8 ptable[128];
@@ -28,25 +28,25 @@ static int depack_skyt(FILE *in, FILE *out)
 	/* read and write sample descriptions */
 	for (i = 0; i < 31; i++) {
 		pw_write_zero(out, 22);			/*sample name */
-		write16b(out, size = read16b(in));	/* sample size */
+		write16b(out, size = hio_read16b(in));	/* sample size */
 		ssize += size * 2;
-		write8(out, read8(in));			/* finetune */
-		write8(out, read8(in));			/* volume */
-		write16b(out, read16b(in));		/* loop start */
-		write16b(out, read16b(in));		/* loop size */
+		write8(out, hio_read8(in));			/* finetune */
+		write8(out, hio_read8(in));			/* volume */
+		write16b(out, hio_read16b(in));		/* loop start */
+		write16b(out, hio_read16b(in));		/* loop size */
 	}
 
-	read32b(in);			/* bypass 8 empty bytes */
-	read32b(in);
-	read32b(in);			/* bypass "SKYT" ID */
+	hio_read32b(in);			/* bypass 8 empty bytes */
+	hio_read32b(in);
+	hio_read32b(in);			/* bypass "SKYT" ID */
 
-	write8(out, pat_pos = read8(in) + 1);	/* pattern table lenght */
+	write8(out, pat_pos = hio_read8(in) + 1);	/* pattern table lenght */
 	write8(out, 0x7f);			/* write NoiseTracker byte */
 
 	/* read track numbers ... and deduce pattern list */
 	for (i = 0; i < pat_pos; i++) {
 		for (j = 0; j < 4; j++)
-			trkval[i][j] = read16b(in);
+			trkval[i][j] = hio_read16b(in);
 	}
 
 	/* write pseudo pattern list */
@@ -55,23 +55,23 @@ static int depack_skyt(FILE *in, FILE *out)
 
 	write32b(out, PW_MOD_MAGIC);		/* write ptk's ID */
 
-	read8(in);				/* bypass $00 unknown byte */
+	hio_read8(in);				/* bypass $00 unknown byte */
 
 	/* get track address */
-	trk_addr = ftell(in);
+	trk_addr = hio_tell(in);
 
 	/* track data */
 	for (i = 0; i < pat_pos; i++) {
 		memset(pat, 0, 1024);
 		for (j = 0; j < 4; j++) {
-			fseek(in, trk_addr + ((trkval[i][j] - 1)<<8), SEEK_SET);
+			hio_seek(in, trk_addr + ((trkval[i][j] - 1)<<8), SEEK_SET);
 			for (k = 0; k < 64; k++) {
 				int x = k * 16 + j * 4;
 
-				c1 = read8(in);
-				c2 = read8(in);
-				c3 = read8(in);
-				c4 = read8(in);
+				c1 = hio_read8(in);
+				c2 = hio_read8(in);
+				c3 = hio_read8(in);
+				c4 = hio_read8(in);
 
 				pat[x] = (c2 & 0xf0) | ptk_table[c1][0];
 				pat[x + 1] = ptk_table[c1][1];
