@@ -11,7 +11,7 @@
 #include "prowiz.h"
 
 
-static int depack_tdd(FILE *in, FILE *out)
+static int depack_tdd(HIO_HANDLE *in, FILE *out)
 {
 	uint8 *tmp;
 	uint8 pat[1024];
@@ -31,7 +31,7 @@ static int depack_tdd(FILE *in, FILE *out)
 	tmp = (uint8 *)malloc(130);
 	memset(tmp, 0, 130);
 	fseek(out, 950, 0);
-	fread(tmp, 130, 1, in);
+	hio_read(tmp, 130, 1, in);
 	fwrite(tmp, 130, 1, out);
 
 	for (pmax = i = 0; i < 128; i++) {
@@ -45,22 +45,22 @@ static int depack_tdd(FILE *in, FILE *out)
 	for (i = 0; i < 31; i++) {
 		fseek(out, 42 + (i * 30), SEEK_SET);
 		/* sample address */
-		saddr[i] = read32b(in);
+		saddr[i] = hio_read32b(in);
 
 		/* read/write size */
-		write16b(out, size = read16b(in));
+		write16b(out, size = hio_read16b(in));
 		ssize += size;
 		ssizes[i] = size;
 
-		write8(out, read8(in));		/* read/write finetune */
-		write8(out, read8(in));		/* read/write volume */
+		write8(out, hio_read8(in));		/* read/write finetune */
+		write8(out, hio_read8(in));		/* read/write volume */
 		/* read/write loop start */
-		write16b(out, (read32b(in) - saddr[i]) / 2);
-		write16b(out, read16b(in));	/* read/write replen */
+		write16b(out, (hio_read32b(in) - saddr[i]) / 2);
+		write16b(out, hio_read16b(in));	/* read/write replen */
 	}
 
 	/* bypass Samples datas */
-	fseek(in, ssize, SEEK_CUR);
+	hio_seek(in, ssize, SEEK_CUR);
 
 	/* write ptk's ID string */
 	fseek(out, 0, SEEK_END);
@@ -71,7 +71,7 @@ static int depack_tdd(FILE *in, FILE *out)
 	for (i = 0; i <= pmax; i++) {
 		memset(tmp, 0, 1024);
 		memset(pat, 0, 1024);
-		fread(tmp, 1024, 1, in);
+		hio_read(tmp, 1024, 1, in);
 		for (j = 0; j < 64; j++) {
 			for (k = 0; k < 4; k++) {
 				int x = j * 16 + k * 4;
@@ -99,7 +99,7 @@ static int depack_tdd(FILE *in, FILE *out)
 	for (i = 0; i < 31; i++) {
 		if (ssizes[i] == 0)
 			continue;
-		fseek(in, saddr[i], SEEK_SET);
+		hio_seek(in, saddr[i], SEEK_SET);
 		pw_move_data(out, in, ssizes[i]);
 	}
 
