@@ -114,12 +114,6 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 			event->vol = *pat++;
 		    if (b & XM_EVENT_FXTYPE_FOLLOWS) {
 			event->fxt = *pat++;
-#if 0
-			if (event->fxt == FX_GLOBALVOL)
-			    event->fxt = FX_TRK_VOL;
-			if (event->fxt == FX_G_VOLSLIDE)
-			    event->fxt = FX_TRK_VSLIDE;
-#endif
 		    }
 		    if (b & XM_EVENT_FXPARM_FOLLOWS)
 			event->fxp = *pat++;
@@ -146,6 +140,15 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		    event->note += 12;
 		}
 
+		if (event->fxt == 0x0e) {
+			switch (event->fxp) {
+			case 0x43:
+			case 0x73:
+				event->fxp--;
+				break;
+			}
+		}
+
 		if (!event->vol)
 		    continue;
 
@@ -185,11 +188,11 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		    event->f2p = (event->vol - 0xc0) << 4;
 		    break;
 		case 0x0d:	/* Pan slide left */
-		    event->f2t = FX_PANSLIDE;
+		    event->f2t = FX_PANSL_NOMEM;
 		    event->f2p = (event->vol - 0xd0) << 4;
 		    break;
 		case 0x0e:	/* Pan slide right */
-		    event->f2t = FX_PANSLIDE;
+		    event->f2t = FX_PANSL_NOMEM;
 		    event->f2p = event->vol - 0xe0;
 		    break;
 		case 0x0f:	/* Tone portamento */
@@ -216,6 +219,15 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 				event->f2p = 0xff;
 			}
 		    }
+
+		    /* From OpenMPT porta-offset.xm:
+		     * "If there is a portamento command next to an offset
+		     *  command, the offset command is ignored completely. In
+		     *  particular, the offset parameter is not memorized."
+		     */
+		    if (event->fxt == FX_OFFSET && event->f2t == FX_TONEPORTA) {
+			event->fxt = event->fxp = 0;
+		    }
 		    break;
 		}
 		event->vol = 0;
@@ -511,6 +523,7 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
     mod->bpm = xfh.bpm;
 
     m->quirk |= xfh.flags & XM_LINEAR_PERIOD_MODE ? QUIRK_LINEAR : 0;
+    m->quirk |= QUIRK_FT2LOOP;
 
     memcpy(mod->xxo, xfh.order, mod->len);
     tracker_name[20] = 0;
@@ -525,24 +538,24 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 #ifndef LIBXMP_CORE_PLAYER
     if (xfh.headersz == 0x0113) {
 	strcpy(tracker_name, "unknown tracker");
+        m->quirk &= ~QUIRK_FT2LOOP;
     } else if (*tracker_name == 0) {
 	strcpy(tracker_name, "Digitrakker");	/* best guess */
+        m->quirk &= ~QUIRK_FT2LOOP;
     }
 
     /* See MMD1 loader for explanation */
     if (!strncmp(tracker_name, "MED2XM by J.Pynnone", 19)) {
 	if (mod->bpm <= 10)
 	    mod->bpm = 125 * (0x35 - mod->bpm * 2) / 33;
+        m->quirk &= ~QUIRK_FT2LOOP;
     }
 
     if (!strncmp(tracker_name, "FastTracker v 2.00", 18)) {
 	strcpy(tracker_name, "old ModPlug Tracker");
+        m->quirk &= ~QUIRK_FT2LOOP;
     }
 
-    if (!strncmp(tracker_name, "MilkyTracker", 12)) {
-	m->quirk |= QUIRK_MLKDLY;
-    }
-	
     set_type(m, "%s XM %d.%02d", tracker_name,
 				xfh.version >> 8, xfh.version & 0xff);
 #else
