@@ -68,6 +68,8 @@ static const struct xmp_event empty_event = { 0, 0, 0, 0, 0, 0, 0 };
  */
 
 
+#ifndef LIBXMP_CORE_PLAYER
+
 /* From http://www.un4seen.com/forum/?topic=7554.0
  *
  * "Invert loop" effect replaces (!) sample data bytes within loop with their
@@ -96,20 +98,18 @@ static void update_invloop(struct module_data *m, struct channel_data *xc)
 		xc->invloop.count = 0;
 		len = xxs->lpe - xxs->lps;	
 
-		if (HAS_QUIRK(QUIRK_FUNKIT)) {
-			/* FIXME: not implemented */
-		} else {
-			if (++xc->invloop.pos > len) {
-				xc->invloop.pos = 0;
-			}
+		if (++xc->invloop.pos > len) {
+			xc->invloop.pos = 0;
+		}
 
-			if (~xxs->flg & XMP_SAMPLE_16BIT) {
-				xxs->data[xxs->lps + xc->invloop.pos] ^= 0xff;
-			}
+		if (~xxs->flg & XMP_SAMPLE_16BIT) {
+			xxs->data[xxs->lps + xc->invloop.pos] ^= 0xff;
 		}
 	}
 }
 
+#endif
+
 static void reset_channels(struct context_data *ctx)
 {
 	struct player_data *p = &ctx->p;
@@ -187,7 +187,7 @@ static int check_delay(struct context_data *ctx, struct xmp_event *e, int chn)
 	if (HAS_QUIRK(QUIRK_RTDELAY)) {
 		if (e->note == 0)
 			xc->delayed_event.note = xc->key + 1;
-		if (e->ins == 0 && !HAS_QUIRK(QUIRK_MLKDLY))
+		if (e->ins == 0)
 			xc->delayed_event.ins = xc->old_ins;
 	}
 
@@ -203,12 +203,6 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 	struct flow_control *f = &p->flow;
 	struct xmp_event *event;
 
-	/* FIXME: workaround for pattern delay + break) */
-	if (f->skip_fetch) {
-		f->skip_fetch = 0;
-		return;
-	}
-
 	for (chn = 0; chn < mod->chn; chn++) {
 		const int num_rows = mod->xxt[TRACK_NUM(pat, chn)]->rows;
 		if (row < num_rows) {
@@ -256,7 +250,7 @@ static inline int get_channel_vol(struct context_data *ctx, int chn)
 
 #define DOENV_RELEASE ((TEST_NOTE(NOTE_RELEASE) || act == VIRT_ACTION_OFF))
 
-static void process_volume(struct context_data *ctx, int chn, int t, int act)
+static void process_volume(struct context_data *ctx, int chn, int act)
 {
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
@@ -327,7 +321,9 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 
 	if (TEST(TREMOLO)) {
 		finalvol += get_lfo(&xc->tremolo.lfo, 1 << 6);
-		update_lfo(&xc->tremolo.lfo);
+		if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
+			update_lfo(&xc->tremolo.lfo);
+		}
 	}
 
 	CLAMP(finalvol, 0, m->volbase);
@@ -385,7 +381,7 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 	virt_setvol(ctx, chn, finalvol);
 }
 
-static void process_frequency(struct context_data *ctx, int chn, int t, int act)
+static void process_frequency(struct context_data *ctx, int chn, int act)
 {
 #ifndef LIBXMP_CORE_DISABLE_IT
 	struct mixer_data *s = &ctx->s;
@@ -442,7 +438,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 			vibrato += vib;
 		}
 
-		if (t % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
+		if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
 			update_lfo(&xc->vibrato.lfo);
 		}
 	}
@@ -523,7 +519,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 #endif
 }
 
-static void process_pan(struct context_data *ctx, int chn, int t, int act)
+static void process_pan(struct context_data *ctx, int chn, int act)
 {
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
@@ -560,7 +556,7 @@ static void process_pan(struct context_data *ctx, int chn, int t, int act)
 	virt_setpan(ctx, chn, finalpan);
 }
 
-static void update_volume(struct context_data *ctx, int chn, int t)
+static void update_volume(struct context_data *ctx, int chn)
 {
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
@@ -569,7 +565,7 @@ static void update_volume(struct context_data *ctx, int chn, int t)
 	/* Volume slides happen in all frames but the first, except when the
 	 * "volume slide on all frames" flag is set.
 	 */
-	if (t % p->speed != 0 || HAS_QUIRK(QUIRK_VSALL)) {
+	if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_VSALL)) {
 		if (TEST(GVOL_SLIDE)) {
 			p->gvol += xc->gvol.slide;
 		}
@@ -599,7 +595,7 @@ static void update_volume(struct context_data *ctx, int chn, int t)
 		}
 	}
 
-	if (t % p->speed == 0) {
+	if (p->frame % p->speed == 0) {
 		/* Process "fine" effects */
 		if (TEST(FINE_VOLS))
 			xc->volume += xc->vol.fslide;
@@ -617,13 +613,13 @@ static void update_volume(struct context_data *ctx, int chn, int t)
 	CLAMP(xc->mastervol, 0, m->volbase);
 }
 
-static void update_frequency(struct context_data *ctx, int chn, int t)
+static void update_frequency(struct context_data *ctx, int chn)
 {
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
 	struct channel_data *xc = &p->xc_data[chn];
 
-	if (t % p->speed != 0 || HAS_QUIRK(QUIRK_PBALL)) {
+	if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_PBALL)) {
 		if (TEST(PITCHBEND) || TEST_PER(PITCHBEND)) {
 			xc->period += xc->freq.slide;
 		}
@@ -651,7 +647,7 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 		} 
 	}
 
-	if (t % p->speed == 0) {
+	if (p->frame % p->speed == 0) {
 		if (TEST(FINE_BEND)) {
 			xc->period += xc->freq.fslide;
 		}
@@ -684,13 +680,13 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 	xc->arpeggio.count %= xc->arpeggio.size;
 }
 
-static void update_pan(struct context_data *ctx, int chn, int t)
+static void update_pan(struct context_data *ctx, int chn)
 {
 	struct player_data *p = &ctx->p;
 	struct channel_data *xc = &p->xc_data[chn];
 
 	if (TEST(PAN_SLIDE)) {
-		if (t % p->speed == 0) {
+		if (p->frame % p->speed == 0) {
 			xc->pan.val += xc->pan.fslide;
 		} else {
 			xc->pan.val += xc->pan.slide;
@@ -704,7 +700,7 @@ static void update_pan(struct context_data *ctx, int chn, int t)
 	}
 }
 
-static void play_channel(struct context_data *ctx, int chn, int t)
+static void play_channel(struct context_data *ctx, int chn)
 {
 	struct player_data *p = &ctx->p;
 	struct smix_data *smix = &ctx->smix;
@@ -731,11 +727,11 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 	act = virt_cstat(ctx, chn);
 	if (act == VIRT_INVALID) {
 		/* We need this to keep processing global volume slides */
-		update_volume(ctx, chn, t);
+		update_volume(ctx, chn);
 		return;
 	}
 
-	if (t == 0 && act != VIRT_ACTIVE) {
+	if (p->frame == 0 && act != VIRT_ACTIVE) {
 		if (!IS_VALID_INSTRUMENT_OR_SFX(xc->ins) || act == VIRT_ACTION_CUT) {
 			virt_resetchannel(ctx, chn);
 			return;
@@ -746,7 +742,7 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 		return;
 
 #ifndef LIBXMP_CORE_PLAYER
-	play_extras(ctx, xc, chn, t);
+	play_extras(ctx, xc, chn);
 #endif
 
 	/* Do cut/retrig */
@@ -773,17 +769,19 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 			SET_NOTE(NOTE_RELEASE);
 	}
 
-	process_volume(ctx, chn, t, act);
-	process_frequency(ctx, chn, t, act);
-	process_pan(ctx, chn, t, act);
+	process_volume(ctx, chn, act);
+	process_frequency(ctx, chn, act);
+	process_pan(ctx, chn, act);
 
-	update_volume(ctx, chn, t);
-	update_frequency(ctx, chn, t);
-	update_pan(ctx, chn, t);
+	update_volume(ctx, chn);
+	update_frequency(ctx, chn);
+	update_pan(ctx, chn);
 
-	if (HAS_QUIRK(QUIRK_INVLOOP)) {
+#ifndef LIBXMP_CORE_PLAYER
+	if (HAS_QUIRK(QUIRK_PROTRACK)) {
 		update_invloop(m, xc);
 	}
+#endif
 
 	xc->info_position = virt_getvoicepos(ctx, chn);
 }
@@ -868,15 +866,6 @@ static void next_row(struct context_data *ctx)
 	struct player_data *p = &ctx->p;
 	struct flow_control *f = &p->flow;
 
-	/* If break during pattern delay, next row is skipped.
-	 * See corruption.mod order 1D (pattern 0D) last line:
-	 * EE2 + D31 ignores D00 in order 1C line 31
-	 * Reported by The Welder <welder@majesty.net>, Jan 14 2012
-	 */
-	if (f->delay && f->pbreak) {
-		f->skip_fetch = 1;
-	}
-
 	p->frame = 0;
 	f->delay = 0;
 
@@ -1123,6 +1112,13 @@ int xmp_play_frame(xmp_context opaque)
 	} else {
 		p->frame++;
 		if (p->frame >= (p->speed * (1 + f->delay))) {
+			/* If break during pattern delay, next row is skipped.
+			 * See corruption.mod order 1D (pattern 0D) last line:
+			 * EE2 + D31 ignores D00 in order 1C line 31. Reported
+			 * by The Welder <welder@majesty.net>, Jan 14 2012
+			 */
+			if (f->delay && f->pbreak)
+				next_row(ctx);
 			next_row(ctx);
 		}
 	}
@@ -1159,7 +1155,7 @@ int xmp_play_frame(xmp_context opaque)
 
 	/* play_frame */
 	for (i = 0; i < p->virt.virt_channels; i++) {
-		play_channel(ctx, i, p->frame);
+		play_channel(ctx, i);
 	}
 
 	p->frame_time = m->time_factor * m->rrate / p->bpm;
