@@ -58,6 +58,7 @@ struct retrig_control {
 #define NOTE_CUT	(1 << 3)
 #define NOTE_ENV_END	(1 << 4)
 #define NOTE_SAMPLE_END	(1 << 5)
+#define NOTE_SET	(1 << 6)	/* for IT portamento after keyoff */
 
 #define IS_VALID_INSTRUMENT(x) ((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0)
 #define IS_VALID_INSTRUMENT_OR_SFX(x) (((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0) || (smix->ins > 0 && (uint32)(x) < mod->ins + smix->ins))
@@ -78,7 +79,6 @@ struct channel_data {
 	int finetune;		/* Guess what */
 	int ins;		/* Instrument number */
 	int old_ins;		/* Last instruemnt */
-	int old_insvol;		/* Last instrument that did set a note */
 	int smp;		/* Sample number */
 	int mastervol;		/* Master vol -- for IT track vol effect */
 	int delay;		/* Note delay in frames */
@@ -88,8 +88,6 @@ struct channel_data {
 	int gliss;		/* Glissando active */
 	int volume;		/* Current volume */
 	int gvl;		/* Global volume for instrument for IT */
-	int offset;		/* Sample offset memory */
-	int offset_val;		/* Sample offset */
 
 	int v_idx;		/* Volume envelope index */
 	int p_idx;		/* Pan envelope index */
@@ -124,6 +122,12 @@ struct channel_data {
 		int sweep;
 	} insvib;
 
+	struct {
+		int val;
+		int val2;	/* For fx9 bug emulation */
+		int memory;
+	} offset;
+
 	struct {
 		int val;	/* Retrig value */
 		int count;	/* Retrig counter */
