@@ -151,6 +151,29 @@ static void set_period(struct context_data *ctx, int note,
 	}
 }
 
+/* From OpenMPT Porta-Pickup.xm:
+ * "An instrument number should not reset the current portamento target. The
+ *  portamento target is valid until a new target is specified by combining a
+ *  note and a portamento effect."
+ */
+static void set_period_ft2(struct context_data *ctx, int note,
+				struct xmp_subinstrument *sub,
+				struct channel_data *xc, int is_toneporta)
+{
+	struct module_data *m = &ctx->m;
+
+	if ((note > 0 && is_toneporta) || xc->porta.target == 0) {
+		xc->porta.target = note_to_period(note, xc->finetune,
+				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+	}
+	if (sub != NULL && note >= 0) {
+		if (xc->period < 1 || !is_toneporta) {
+			xc->period = note_to_period(note, xc->finetune,
+				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+		}
+	}
+}
+
 
 #ifndef LIBXMP_CORE_PLAYER
 #define IS_TONEPORTA(x) ((x) == FX_TONEPORTA || (x) == FX_TONE_VSLIDE \
@@ -191,7 +214,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		SET(NEW_INS);
 		xc->fadeout = 0x10000;	/* for painlace.mod pat 0 ch 3 echo */
 		xc->per_flags = 0;
-		xc->offset_val = 0;
+		xc->offset.val = 0;
 		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins)) {
@@ -275,10 +298,13 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 
 	if (note >= 0) {
 		xc->note = note;
+		virt_voicepos(ctx, chn, xc->offset.val);
+	}
 
-		virt_voicepos(ctx, chn, xc->offset_val);
-		if (TEST(OFFSET) && p->flags & XMP_FLAGS_FX9BUG)
-			xc->offset_val <<= 1;
+	if (TEST(OFFSET)) {
+		if (HAS_QUIRK(QUIRK_PROTRACK) || p->flags & XMP_FLAGS_FX9BUG) {
+			xc->offset.val += xc->offset.val2;
+		}
 		RESET(OFFSET);
 	}
 
@@ -347,6 +373,24 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		ins = 0;
 	}
 
+	/* FT2: Retrieve old instrument volume */
+	if (ins) {
+		if (key == 0 || key >= XMP_KEY_OFF) {
+			struct xmp_subinstrument *sub;
+
+			/* Previous instrument */
+			sub = get_subinstrument(ctx, xc->ins, xc->key);
+
+			/* No note */
+			if (sub != NULL) {
+				xc->volume = sub->vol;
+				xc->pan.val = sub->pan;
+				xc->ins_fade = mod->xxi[xc->ins].rls;
+				SET(NEW_VOL);
+			}
+		}
+	}
+
 	/* Do this regardless if the instrument is invalid or not */
 	if (ev.ins) {
 		SET(NEW_INS);
@@ -378,22 +422,10 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 	/* Check note */
 
-	/* FT2: Retrieve old instrument volume */
 	if (ins) {
-		struct xmp_subinstrument *sub;
+		if (key > 0 && key < XMP_KEY_OFF) {
+			struct xmp_subinstrument *sub;
 
-		if (key == 0 || key >= XMP_KEY_OFF) {
-			/* Previous instrument */
-			sub = get_subinstrument(ctx, xc->old_insvol, xc->key);
-
-			/* No note */
-			if (sub != NULL) {
-				xc->volume = sub->vol;
-				xc->pan.val = sub->pan;
-				xc->ins_fade = mod->xxi[xc->old_insvol].rls;
-				SET(NEW_VOL);
-			}
-		} else {
 			/* Retrieve volume when we have note */
 
 			/* and only if we have instrument, otherwise we're in
@@ -409,7 +441,6 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			} else {
 				xc->volume = 0;
 			}
-			xc->old_insvol = xc->ins;
 			SET(NEW_VOL);
 		}
 	}
@@ -543,12 +574,12 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* FT2: always reset sample offset */
-	xc->offset_val = 0;
+	xc->offset.val = 0;
 
 	/* Secondary effect handled first */
 	process_fx(ctx, xc, chn, &ev, 1);
 	process_fx(ctx, xc, chn, &ev, 0);
-	set_period(ctx, note, sub, xc, is_toneporta);
+	set_period_ft2(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
 		use_ins_vol = 0;
@@ -565,7 +596,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		 *  stop playback on this channel."
 		 */
 
-		if (xc->offset_val >= mod->xxs[sub->sid].len) {
+		if (xc->offset.val >= mod->xxs[sub->sid].len) {
 			virt_resetchannel(ctx, chn);
 		} else {
 
@@ -574,7 +605,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			 * without tone portamento, otherwise we won't play
 			 * sweeps and loops correctly.
 			 */
-			virt_voicepos(ctx, chn, xc->offset_val);
+			virt_voicepos(ctx, chn, xc->offset.val);
 		}
 	}
 
@@ -620,7 +651,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 		use_ins_vol = 1;
 		xc->fadeout = 0x10000;
 		xc->per_flags = 0;
-		xc->offset_val = 0;
+		xc->offset.val = 0;
 		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins)) {
@@ -661,7 +692,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 			 * 7spirits.s3m, mod.Biomechanoid
 			 */
 			if (not_same_ins) {
-				xc->offset_val = 0;
+				xc->offset.val = 0;
 			}
 		} else {
 			xc->key = e->note - 1;
@@ -718,7 +749,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 
 	if (note >= 0) {
 		xc->note = note;
-		virt_voicepos(ctx, chn, xc->offset_val);
+		virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
 	if (use_ins_vol) {
@@ -774,16 +805,24 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	candidate_ins = xc->ins;
 
 	/* Notes with unmapped instruments are ignored */
-	if (ev.ins && ev.ins <= mod->ins && ev.note && ev.note <= XMP_MAX_KEYS) {
-		int ins = ev.ins - 1;
-		int key = ev.note - 1;
+	if (ev.ins) {
+		if (ev.ins <= mod->ins && ev.note && ev.note <= XMP_MAX_KEYS) {
+			int ins = ev.ins - 1;
+			int key = ev.note - 1;
 
-		if (ins < mod->ins) {
-			int smp = mod->xxi[ins].map[key].ins;
-			if (smp == 0xff || smp >= mod->smp) {
-				candidate_ins = ins;
-				memset(&ev, 0, sizeof(struct xmp_event));
-			};
+			if (ins < mod->ins) {
+				int smp = mod->xxi[ins].map[key].ins;
+				if (smp == 0xff || smp >= mod->smp) {
+					candidate_ins = ins;
+					memset(&ev, 0, sizeof (ev));
+				};
+			}
+		}
+	} else {
+		if (ev.note && ev.note <= XMP_MAX_KEYS) {
+			if (!IS_VALID_INSTRUMENT(xc->old_ins - 1)) {
+				new_invalid_ins = 1;
+			}
 		}
 	}
 
@@ -799,7 +838,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		is_toneporta = 0;
 	}
 
-	if (xc->period <= 0) {
+	if (xc->period <= 0 || TEST_NOTE(NOTE_END)) {
 		is_toneporta = 0;
 	}
 
@@ -807,8 +846,41 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	if (ev.ins) {
 		int ins = ev.ins - 1;
+		int set_new_ins = 0;
 
-		if (!is_release || (!is_toneporta || xc->ins != ins)) {
+		/* portamento_after_keyoff.it test case */
+		if (is_release && is_toneporta && !key) {
+			if (HAS_QUIRK(QUIRK_PRENV) || TEST_NOTE(NOTE_SET)) {
+				is_toneporta = 0;
+				reset_envelopes(ctx, xc, 0);
+			}
+		}
+
+		if (!is_release) {
+			set_new_ins = 1;
+		}
+
+		if (is_toneporta) {
+			if (xc->ins != ins) {
+				set_new_ins = 1;
+			} else if (!HAS_QUIRK(QUIRK_PRENV)) {
+				/* also set if mapped sample is different,
+				 * see OpenMPT PortaInsNum.it
+				 */
+				struct xmp_subinstrument *s1, *s2;
+				s1 = get_subinstrument(ctx, ins, key - 1);
+				s2 = get_subinstrument(ctx, ins, xc->note);
+				if (s1 && s2 && s1->sid != s2->sid) {
+					set_new_ins = 1;
+					/* not same sample, actually */
+					not_same_ins = 1;
+				}
+			}
+		} else {
+			set_new_ins = 1;
+		}
+
+		if (set_new_ins) {
 			SET(NEW_INS);
 			use_ins_vol = 1;
 			reset_env = 1;
@@ -826,16 +898,25 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 				} else {
 					key = xc->key + 1;
 				}
+
+				RESET_NOTE(NOTE_SET);
 			}
 			if (xc->ins != ins && (!is_toneporta || !HAS_QUIRK(QUIRK_PRENV))) {
-				not_same_ins = 1;
+				struct xmp_subinstrument *s1, *s2;
+				s1 = get_subinstrument(ctx, xc->ins, xc->key);
+				s2 = get_subinstrument(ctx, ins, key - 1);
+
 				candidate_ins = ins;
-				if (is_toneporta) {
-					/* Get new instrument volume */
-					sub = get_subinstrument(ctx, ins, key);
-					if (sub != NULL) {
-						xc->volume = sub->vol;
-						use_ins_vol = 0;
+
+				if (!s1 || !s2 || s1->sid != s2->sid) {
+					not_same_ins = 1;
+					if (is_toneporta) {
+						/* Get new instrument volume */
+						sub = get_subinstrument(ctx, ins, key);
+						if (sub != NULL) {
+							xc->volume = sub->vol;
+							use_ins_vol = 0;
+						}
 					}
 				}
 			}
@@ -851,6 +932,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	if (key && !new_invalid_ins) {
 		SET(NEW_NOTE);
+		SET_NOTE(NOTE_SET);
 
 		if (key == XMP_KEY_FADE) {
 			SET_NOTE(NOTE_FADEOUT);
@@ -864,18 +946,20 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			SET_NOTE(NOTE_RELEASE);
 			reset_env = 0;
 			use_ins_vol = 0;
-			if (HAS_QUIRK(QUIRK_PRENV))
-				SET_NOTE(NOTE_END);
-		} else if (is_toneporta) {
+		} else {
+			/* portamento_after_keyoff.it test case */
+			reset_env = 1;
 
-			/* Always retrig on tone portamento: Fix portamento in
-			 * 7spirits.s3m, mod.Biomechanoid
-			 */
-			if (not_same_ins || TEST_NOTE(NOTE_END)) {
-				SET(NEW_INS);
-				RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
-			} else {
-				key = 0;
+			if (is_toneporta) {
+				/* Always retrig on tone portamento: Fix
+				 * portamento in 7spirits.s3m, mod.Biomechanoid
+			 	 */
+				if (not_same_ins || TEST_NOTE(NOTE_END)) {
+					SET(NEW_INS);
+					RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
+				} else {
+					key = 0;
+				}
 			}
 		}
 	}
@@ -962,7 +1046,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 
 	/* IT: always reset sample offset */
-	xc->offset_val = 0;
+	xc->offset.val = 0;
 
 	/* According to Storlek test 25, Impulse Tracker handles the volume
 	 * column effects last.
@@ -981,11 +1065,13 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	if (note >= 0) {
 		xc->note = note;
-		virt_voicepos(ctx, chn, xc->offset_val);
+		virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
 	if (reset_env) {
-		RESET_NOTE(NOTE_RELEASE | NOTE_FADEOUT);
+		if (ev.note) {
+			RESET_NOTE(NOTE_RELEASE | NOTE_FADEOUT);
+		}
 		/* Set after copying to new virtual channel (see ambio.it) */
 		xc->fadeout = 0x10000;
 	}
@@ -1031,7 +1117,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 		use_ins_vol = 1;
 		SET(NEW_INS);
 		xc->fadeout = 0x10000;
-		xc->offset_val = 0;
+		xc->offset.val = 0;
 		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins)) {
@@ -1146,7 +1232,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 
 	if (note >= 0) {
 		xc->note = note;
-		virt_voicepos(ctx, chn, xc->offset_val);
+		virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
 	if (use_ins_vol) {
@@ -1180,7 +1266,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	SET(NEW_INS);
 	xc->fadeout = 0x10000;
 	xc->per_flags = 0;
-	xc->offset_val = 0;
+	xc->offset.val = 0;
 	RESET_NOTE(NOTE_RELEASE);
 
 	xc->ins = ins;
@@ -1237,7 +1323,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	SET(NEW_VOL);
 
 	xc->note = note;
-	virt_voicepos(ctx, chn, xc->offset_val);
+	virt_voicepos(ctx, chn, xc->offset.val);
 
 	return 0;
 }
