@@ -54,13 +54,13 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     struct module_data *m = &ctx->m;
     struct xmp_module *mod = &m->mod;
     int parm, gvol_memory, f1, f2, p1, p2, ord, ord2;
-    int row, last_row, break_row, cnt_row;
+    int row, last_row, break_row, row_count;
     int gvl, bpm, speed, base_time, chn;
-    int alltmp;
-    double clock, clock_rst;
-    int loop_chn, loop_flg;
-    int pdelay = 0, skip_fetch;
-    int loop_stk[XMP_MAX_CHANNELS];
+    int frame_count;
+    double time, start_time;
+    int loop_chn, loop_count;
+    int pdelay = 0;
+    int loop_stack[XMP_MAX_CHANNELS];
     int loop_row[XMP_MAX_CHANNELS];
     struct xmp_event* event;
     int i, pat;
@@ -78,9 +78,10 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 			mod->xxp[pat]->rows ? mod->xxp[pat]->rows : 1);
     }
 
-    memset(loop_stk, 0, sizeof(int) * mod->chn);
+    memset(loop_stack, 0, sizeof(int) * mod->chn);
     memset(loop_row, 0, sizeof(int) * mod->chn);
-    loop_chn = loop_flg = 0;
+    loop_count = 0;
+    loop_chn = -1;
 
     gvl = mod->gvl;
     bpm = mod->bpm;
@@ -110,9 +111,8 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     ord2 = -1;
     ord = ep - 1;
 
-    gvol_memory = break_row = cnt_row = alltmp = 0;
-    clock_rst = clock = 0.0;
-    skip_fetch = 0;
+    gvol_memory = break_row = row_count = frame_count = 0;
+    start_time = time = 0.0;
 
     while (42) {
 	if ((uint32)++ord >= mod->len) {
@@ -152,12 +152,15 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 	if (break_row < mod->xxp[pat]->rows && m->scan_cnt[ord][break_row]) {
 	    break;
+	} else if (break_row >= mod->xxp[pat]->rows) {
+	    break_row = 0;
 	}
 
+
 	info->gvl = gvl;
 	info->bpm = bpm;
 	info->speed = speed;
-	info->time = clock + m->time_factor * alltmp / bpm;
+	info->time = time + m->time_factor * frame_count * base_time / bpm;
 
 #ifndef LIBXMP_CORE_PLAYER
 	info->st26_speed = st26_speed;
@@ -165,14 +168,14 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 	if (info->start_row == 0 && ord != 0) {
 	    if (ord == ep) {
-		clock_rst = clock + m->time_factor * alltmp / bpm;
+		start_time = time + m->time_factor * frame_count * base_time / bpm;
 	    }
 
 	    info->start_row = break_row;
 	}
 
 	last_row = mod->xxp[pat]->rows;
-	for (row = break_row, break_row = 0; row < last_row; row++, cnt_row++) {
+	for (row = break_row, break_row = 0; row < last_row; row++, row_count++) {
 	    /* Prevent crashes caused by large softmixer frames */
 	    if (bpm < XMP_MIN_BPM) {
 	        bpm = XMP_MIN_BPM;
@@ -191,11 +194,11 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	     * (...) it dies at the end of position 2F
 	     */
 
-	    if (cnt_row > 512)	/* was 255, but Global trash goes to 318 */
+	    if (row_count > 512)	/* was 255, but Global trash goes to 318 */
 		goto end_module;
 
-	    if (!loop_flg && m->scan_cnt[ord][row]) {
-		cnt_row--;
+	    if (!loop_count && m->scan_cnt[ord][row]) {
+		row_count--;
 		goto end_module;
 	    }
 	    m->scan_cnt[ord][row]++;
@@ -208,17 +211,10 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 		event = &EVENT(mod->xxo[ord], chn, row);
 
-		/* Pattern delay + pattern break cause target row events
-		 * to be ignored
-		 */
-		if (skip_fetch) {
-		    f1 = p1 = f2 = p2 = 0;
-		} else {
-		    f1 = event->fxt;
-		    p1 = event->fxp;
-		    f2 = event->f2t;
-		    p2 = event->f2p;
-		}
+		f1 = event->fxt;
+		p1 = event->fxp;
+		f2 = event->f2t;
+		p2 = event->f2p;
 
 		if (f1 == FX_GLOBALVOL || f2 == FX_GLOBALVOL) {
 		    gvl = (f1 == FX_GLOBALVOL) ? p1 : p2;
@@ -263,8 +259,8 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 		if ((f1 == FX_SPEED && p1) || (f2 == FX_SPEED && p2)) {
 		    parm = (f1 == FX_SPEED) ? p1 : p2;
-		    alltmp += cnt_row * speed * base_time;
-		    cnt_row = 0;
+		    frame_count += row_count * speed;
+		    row_count = 0;
 		    if (parm) {
 			if (p->flags & XMP_FLAGS_VBLANK || parm < 0x20) {
 			    speed = parm;
@@ -272,8 +268,8 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 			    st26_speed = 0;
 #endif
 			} else {
-			    clock += m->time_factor * alltmp / bpm;
-			    alltmp = 0;
+			    time += m->time_factor * frame_count * base_time / bpm;
+			    frame_count = 0;
 			    bpm = parm;
 			}
 		    }
@@ -296,8 +292,8 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 		if ((f1 == FX_S3M_SPEED && p1) || (f2 == FX_S3M_SPEED && p2)) {
 		    parm = (f1 == FX_S3M_SPEED) ? p1 : p2;
-		    alltmp += cnt_row * speed * base_time;
-		    cnt_row  = 0;
+		    frame_count += row_count * speed;
+		    row_count  = 0;
 		    speed = parm;
 #ifndef LIBXMP_CORE_PLAYER
 		    st26_speed = 0;
@@ -307,10 +303,10 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		if ((f1 == FX_S3M_BPM && p1) || (f2 == FX_S3M_BPM && p2)) {
 		    parm = (f1 == FX_S3M_BPM) ? p1 : p2;
 		    if (parm >= 0x20) {
-			alltmp += cnt_row * speed * base_time;
-			cnt_row = 0;
-			clock += m->time_factor * alltmp / bpm;
-			alltmp = 0;
+			frame_count += row_count * speed;
+			row_count = 0;
+			time += m->time_factor * frame_count * base_time / bpm;
+			frame_count = 0;
 			bpm = parm;
 		    }
 		}
@@ -318,43 +314,49 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 #ifndef LIBXMP_CORE_DISABLE_IT
 		if ((f1 == FX_IT_BPM && p1) || (f2 == FX_IT_BPM && p2)) {
 		    parm = (f1 == FX_IT_BPM) ? p1 : p2;
-		    alltmp += cnt_row * speed * base_time;
-		    cnt_row = 0;
-		    clock += m->time_factor * alltmp / bpm;
-		    alltmp = 0;
+		    frame_count += row_count * speed;
+		    row_count = 0;
+		    time += m->time_factor * frame_count * base_time / bpm;
+		    frame_count = 0;
 
 		    if (MSN(parm) == 0) {
-		        clock += m->time_factor * base_time / bpm;
+		        time += m->time_factor * base_time / bpm;
 			for (i = 1; i < speed; i++) {
 			    bpm -= LSN(parm);
 			    if (bpm < 0x20)
 				bpm = 0x20;
-		            clock += m->time_factor * base_time / bpm;
+		            time += m->time_factor * base_time / bpm;
 			}
 
 			/* remove one row at final bpm */
-			clock -= m->time_factor * speed * base_time / bpm;
+			time -= m->time_factor * speed * base_time / bpm;
 
 		    } else if (MSN(parm) == 1) {
-		        clock += m->time_factor * base_time / bpm;
+		        time += m->time_factor * base_time / bpm;
 			for (i = 1; i < speed; i++) {
 			    bpm += LSN(parm);
 			    if (bpm > 0xff)
 				bpm = 0xff;
-		            clock += m->time_factor * base_time / bpm;
+		            time += m->time_factor * base_time / bpm;
 			}
 
 			/* remove one row at final bpm */
-			clock -= m->time_factor * speed * base_time / bpm;
+			time -= m->time_factor * speed * base_time / bpm;
 
 		    } else {
 			bpm = parm;
 		    }
 		}
+
+		if (f1 == FX_IT_ROWDELAY) {
+	    		m->scan_cnt[ord][row] += p1 & 0x0f;
+			frame_count += (p1 & 0x0f) * speed;
+		}
 #endif
 
 		if (f1 == FX_JUMP || f2 == FX_JUMP) {
 		    ord2 = (f1 == FX_JUMP) ? p1 : p2;
+		    break_row = 0;
 		    last_row = 0;
 		}
 
@@ -369,43 +371,46 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 		    if ((parm >> 4) == EX_PATT_DELAY) {
 			pdelay = parm & 0x0f;
-			alltmp += pdelay * speed * base_time;
+			frame_count += pdelay * speed;
 		    }
 
 		    if ((parm >> 4) == EX_PATTERN_LOOP) {
 			if (parm &= 0x0f) {
-			    if (loop_stk[chn]) {
-				if (--loop_stk[chn])
-				    loop_chn = chn + 1;
+			    /* Loop end */
+			    if (loop_stack[chn]) {
+				if (--loop_stack[chn])
+				    loop_chn = chn;
 				else {
-				    loop_flg--;
+				    loop_count--;
 				    if (m->quirk & QUIRK_S3MLOOP)
-					loop_row[chn] = row + 1;
+					loop_row[chn] = row;
 				}
 			    } else {
-				if (loop_row[chn] <= row) {
-				    loop_stk[chn] = parm;
-				    loop_chn = chn + 1;
-				    loop_flg++;
+				if (loop_row[chn] < row) {
+				    loop_stack[chn] = parm;
+				    loop_chn = chn;
+				    loop_count++;
 				}
 			    }
 			} else { 
-			    loop_row[chn] = row;
+			    /* Loop start */
+			    loop_row[chn] = row - 1;
+			    if (HAS_QUIRK(QUIRK_FT2LOOP))
+				break_row = row;
 			}
 		    }
 		}
 	    }
-	    skip_fetch = 0;
 
-	    if (loop_chn) {
-		row = loop_row[--loop_chn] - 1;
-		loop_chn = 0;
+	    if (loop_chn >= 0) {
+		row = loop_row[loop_chn];
+		loop_chn = -1;
 	    }
 
 #ifndef LIBXMP_CORE_PLAYER
 	    if (st26_speed) {
-	        alltmp += cnt_row * speed * base_time;
-	        cnt_row  = 0;
+	        frame_count += row_count * speed;
+	        row_count  = 0;
 		if (st26_speed & 0x10000) {
 			speed = (st26_speed & 0xff00) >> 8;
 		} else {
@@ -417,7 +422,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	}
 
 	if (break_row && pdelay) {
-	    skip_fetch = 1;
+	    break_row++;
 	}
 
 	if (ord2 >= 0) {
@@ -425,8 +430,8 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	    ord2 = -1;
 	}
 
-	alltmp += cnt_row * speed * base_time;
-	cnt_row = 0;
+	frame_count += row_count * speed;
+	row_count = 0;
     }
     row = break_row;
 
@@ -435,10 +440,10 @@ end_module:
     p->scan[chain].row = row;
     p->scan[chain].ord = ord;
 
-    clock -= clock_rst;
-    alltmp += cnt_row * speed * base_time;
+    time -= start_time;
+    frame_count += row_count * speed;
 
-    return (clock + m->time_factor * alltmp / bpm);
+    return (time + m->time_factor * frame_count * base_time / bpm);
 }
 
 int get_sequence(struct context_data *ctx, int ord)
